/*
 * Copyright (c) 2008 - 2012, Andy Bierman, All Rights Reserved.
 * Copyright (c) 2012 - 2020, YumaWorks, Inc., All Rights Reserved.
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 *

*** Generated by yangdump-sdk
    Combined SIL-SA module
    module silsa-sethook-example
    revision 2020-08-18
    namespace http://netconfcentral.org/ns/silsa-sethook-example
    Created: 2020-08-18T22:38:35Z
    CLI parameters:
        format c
        indent 4
        log-level debug3
        module silsa-sethook-example
        output silsa-sethook-example.c
        sil-get2
        sil-sa
        unified true

 */

#include <xmlstring.h>

#include "procdefs.h"
#include "agt.h"
#include "agt_action.h"
#include "agt_cb.h"
#include "agt_rpc.h"
#include "agt_sil_lib.h"
#include "agt_timer.h"
#include "agt_util.h"
#include "agt_val.h"
#include "cfg.h"
#include "dlq.h"
#include "getcb.h"
#include "ncx.h"
#include "ncx_feature.h"
#include "ncxmod.h"
#include "ncxtypes.h"
#include "obj.h"
#include "rpc.h"
#include "sil_sa.h"
#include "status.h"
#include "val.h"
#include "val_child.h"
#include "val_util.h"
#include "xml_val.h"
#include "silsa-sethook-example.h"

/* module static variables */
static ncx_module_t *silsa_sethook_example_mod;

/* put your static variables here */


/********************************************************************
* FUNCTION print_callback_info
*
* Dump current Callback info
*
********************************************************************/
static void
    print_callback_info (val_value_t *useval,
                         agt_cbtyp_t cbtyp,
                         op_editop_t editop,
                         const xmlChar *callback_name)
{
    if (LOGDEBUG) {
        log_debug("\n (%s) Enter %s callback"
            "\n ------PHASE:%s;EDITOP:%s\n",
            useval ? VAL_NAME(useval) : NCX_NT_NONE,
            callback_name,
            agt_cbtype_name(cbtyp),
            op_editop_name(editop));
    }

} /* print_callback_info */


/********************************************************************
* FUNCTION y_silsa_sethook_example_init_static_vars
*
* initialize module static variables
*
********************************************************************/
static void y_silsa_sethook_example_init_static_vars (void)
{
    silsa_sethook_example_mod = NULL;

    /* init your static variables here */

} /* y_silsa_sethook_example_init_static_vars */


/********************************************************************
* FUNCTION silsa_sethook_example_edit
*
* Edit database object callback
*
* Path: leaf /interfaces/interface/speed
* Path: leaf /interfaces/interface/hook-node
* Path: leaf /interfaces/interface/state/admin-state
* Path: container /interfaces/interface/state
* Path: list /interfaces/interface
* Path: leaf /interfaces/status
* Path: container /interfaces
* Path: leaf /trigger
*
* Add object instrumentation in COMMIT phase.
*
* INPUTS:
*     see agt/agt_cb.h for details
*
* RETURNS:
*     error status
********************************************************************/
static status_t
    silsa_sethook_example_edit (ses_cb_t *scb,
                                rpc_msg_t *msg,
                                agt_cbtyp_t cbtyp,
                                op_editop_t editop,
                                val_value_t *newval,
                                val_value_t *curval)
{
    status_t res = NO_ERR;
    val_value_t *errorval = (curval) ? curval : newval;
    uint32 cur_errs = agt_get_error_count(msg);

    print_callback_info(errorval, cbtyp, editop, (const xmlChar *)"EDIT");

    switch (cbtyp) {
    case AGT_CB_VALIDATE:
        /* description-stmt validation here */
        break;
    case AGT_CB_APPLY:
        /* database manipulation done here */
        break;
    case AGT_CB_COMMIT:
        /* device instrumentation done here */
        switch (editop) {
        case OP_EDITOP_LOAD:
            break;
        case OP_EDITOP_MERGE:
            break;
        case OP_EDITOP_REPLACE:
            break;
        case OP_EDITOP_CREATE:
            break;
        case OP_EDITOP_DELETE:
            break;
        default:
            res = SET_ERROR(ERR_INTERNAL_VAL);
        }
        break;
    case AGT_CB_ROLLBACK:
        /* undo device instrumentation here */
        break;
    default:
        res = SET_ERROR(ERR_INTERNAL_VAL);
    }

    if ((res != NO_ERR) && (cur_errs == agt_get_error_count(msg))) {
        agt_record_error(scb,
                         &msg->mhdr,
                         NCX_LAYER_CONTENT,
                         res,
                         NULL,
                         (errorval) ? NCX_NT_VAL : NCX_NT_NONE,
                         errorval,
                         (errorval) ? NCX_NT_VAL : NCX_NT_NONE,
                         errorval);
    }
    return res;

} /* silsa_sethook_example_edit */


/********************************************************************
* FUNCTION create_list_entry
*
* Make a list entry based on the key
*
* INPUTS:
*   parentobj == parent object template to use
*   keyname == value of the key
*   res = return status
*
* RETURNS:
*    val_value_t of listval entry if no error
*    else NULL
*
*********************************************************************/
static val_value_t *
    create_list_entry(obj_template_t *parentobj,
                      const xmlChar *keyname,
                      status_t *res)
{
    /* get the obj template for the list obj */
    obj_template_t *list_obj =
        obj_find_child(parentobj,
                       y_silsa_sethook_example_M_silsa_sethook_example,
                       (const xmlChar *)"interface");
    if (!list_obj) {
        *res = ERR_NCX_INVALID_VALUE;
        return NULL;
    }

    /* add all the /container/list nodes */
    val_value_t *list_value = val_new_value();
    if (!list_value) {
        *res = ERR_NCX_INVALID_VALUE;
        return NULL;
    }
    val_init_from_template(list_value, list_obj);

    /* make key leaf entry */
    val_value_t *child =
        agt_make_leaf(list_obj,
                      (const xmlChar *)"name",
                       keyname,
                       res);
    if (!child) {
        val_free_value(list_value);
        *res = ERR_NCX_INVALID_VALUE;
        return NULL;
    }
    *res = val_child_add(child, list_value);
    if (*res != NO_ERR) {
        val_free_value(list_value);
        return NULL;
    }

    /* make some extra leaf entries */
    child =
        agt_make_uint_leaf(list_obj,
                           (const xmlChar *)"hook-node",
                           (uint32)1000,
                           res);
    if (!child) {
        val_free_value(list_value);
        *res = ERR_NCX_INVALID_VALUE;
        return NULL;
    }
    *res = val_child_add(child, list_value);
    if (*res != NO_ERR) {
        val_free_value(child);
        val_free_value(list_value);
        return NULL;
    }

    /* generate the internal index Q chain */
    *res = val_gen_index_chain(list_obj, list_value);
    if (*res != NO_ERR) {
        log_error("\nError: could not generate index chain (%s)",
            get_error_string(*res));
        val_free_value(list_value);
        return NULL;
    }

    return list_value;

} /* create_list_entry */


/********************************************************************
* FUNCTION  sethook_callback
*
* Callback function for server object handler
* Used to provide a callback for a specific named object
*
* Set Hook:
*   trigger: edit /trigger
*   add_edit:
*       add nodes: populate 1 list entry with name=vlan1
*
*           path: /interfaces/interface[name=vlan1]
*
*********************************************************************/
static status_t
    sethook_callback (ses_cb_t *scb,
                      rpc_msg_t *msg,
                      op_editop_t editop,
                      val_value_t *newval,
                      val_value_t *curval,
                      const xmlChar *transaction_id,
                      boolean isvalidate,
                      boolean isload,
                      boolean isrunning)
{
    status_t res = NO_ERR;
    val_value_t *errorval = (curval) ? curval : newval;

    const xmlChar *user = sil_sa_get_username();
    const xmlChar *client_addr = sil_sa_get_client_addr();

    if (LOGDEBUG2) {
        log_debug2("\n\n********************************************");

        print_callback_info(errorval,
                            AGT_CB_VALIDATE,
                            editop,
                            (const xmlChar *)"SETHOOK");

        log_debug2("\ntransaction_id -- %s", transaction_id);
        log_debug2("\nuser_id -- %s", user);
        log_debug2("\nclient_addr -- %s", client_addr);
        log_debug2("\nisvalidate -- %s",
            isvalidate ? NCX_EL_TRUE : NCX_EL_FALSE);
        log_debug2("\nisload -- %s",
            isload ? NCX_EL_TRUE : NCX_EL_FALSE);
        log_debug2("\nisrunning -- %s",
            isrunning ? NCX_EL_TRUE : NCX_EL_FALSE);
        log_debug2("\n********************************************\n\n");
    }

    const xmlChar *defpath =
        (const xmlChar *)"/sa-sethook-ex:interfaces";

    switch (editop) {
    case OP_EDITOP_LOAD:
        break;
    case OP_EDITOP_MERGE:
    case OP_EDITOP_REPLACE:
    case OP_EDITOP_CREATE:
        /* add a new edit if the "/trigger" value is "add-edit" */
        if (newval &&
            !xml_strcmp(VAL_STR(newval), (const xmlChar *)"add-edit")) {

            /* find object template of the desired node */
            obj_template_t *targobj =
                ncx_find_object(silsa_sethook_example_mod,
                                (const xmlChar *)"interfaces");
            if (!targobj) {
                return ERR_NCX_INVALID_VALUE;
            }

           /* create edit_value container value */
            val_value_t *editval = val_new_value();
            if (editval == NULL) {
                return ERR_INTERNAL_MEM;
            }
            val_init_from_template(editval, targobj);

            /* malloc and construct list value */
            val_value_t *list_value =
                  create_list_entry(VAL_OBJ(editval),
                                    (const xmlChar *)"vlan1",
                                    &res);
            if (!list_value) {
                val_free_value(editval);
                return res;
            }

            /* add a new list entry */
            res = val_child_add(list_value, editval);
            if (res != NO_ERR) {
                val_free_value(list_value);
            } else {
                /* add a new edit, MERGE on defpath with 1 new list entry */
                const xmlChar *edit_operation = (const xmlChar *)"merge";
                const xmlChar *insert_point = NULL;
                const xmlChar *insert_where = NULL;
                boolean skip_cb = FALSE;

                res =
                    sil_sa_add_edit(defpath,
                                    editval,
                                    edit_operation,
                                    insert_where,
                                    insert_point,
                                    skip_cb);
            }

            /* clean up the editval */
            val_free_value(editval);
        }
        break;
    case OP_EDITOP_DELETE:
        break;
    default:
        res = SET_ERROR(ERR_INTERNAL_VAL);
    }

    if (res != NO_ERR) {
        agt_record_error(scb,
                         &msg->mhdr,
                         NCX_LAYER_CONTENT,
                         res,
                         NULL,
                         (errorval) ? NCX_NT_VAL : NCX_NT_NONE,
                         errorval,
                         (errorval) ? NCX_NT_VAL : NCX_NT_NONE,
                         errorval);
    }

    return res;

}  /* sethook_callback */


/********************************************************************
* FUNCTION y_silsa_sethook_example_init
*
* initialize the silsa-sethook-example server instrumentation library
*
* INPUTS:
*    modname == requested module name
*    revision == requested version (NULL for any)
*
* RETURNS:
*     error status
********************************************************************/
status_t
    y_silsa_sethook_example_init (const xmlChar *modname,
                                  const xmlChar *revision)
{
    status_t res = NO_ERR;

    y_silsa_sethook_example_init_static_vars();

    /* change if custom handling done */
    if (xml_strcmp(modname, y_silsa_sethook_example_M_silsa_sethook_example)) {
        return ERR_NCX_UNKNOWN_MODULE;
    }

    if (revision && xml_strcmp(revision, y_silsa_sethook_example_R_silsa_sethook_example)) {
        return ERR_NCX_WRONG_VERSION;
    }

    res =
        ncxmod_load_module(y_silsa_sethook_example_M_silsa_sethook_example,
                           y_silsa_sethook_example_R_silsa_sethook_example,
                           agt_get_savedevQ(),
                           &silsa_sethook_example_mod);
    if (res != NO_ERR) {
        return res;
    }

    res =
        agt_cb_register_callback(y_silsa_sethook_example_M_silsa_sethook_example,
                                 (const xmlChar *)"/sa-sethook-ex:interfaces",
                                 y_silsa_sethook_example_R_silsa_sethook_example,
                                 silsa_sethook_example_edit);
    if (res != NO_ERR) {
        return res;
    }

    res =
        agt_cb_register_callback(y_silsa_sethook_example_M_silsa_sethook_example,
                    (const xmlChar *)"/sa-sethook-ex:interfaces/sa-sethook-ex:interface",
                                 y_silsa_sethook_example_R_silsa_sethook_example,
                                 silsa_sethook_example_edit);
    if (res != NO_ERR) {
        return res;
    }

    res =
        agt_cb_register_callback(y_silsa_sethook_example_M_silsa_sethook_example,
        (const xmlChar *)"/sa-sethook-ex:interfaces/sa-sethook-ex:interface/sa-sethook-ex:speed",
                                 y_silsa_sethook_example_R_silsa_sethook_example,
                                 silsa_sethook_example_edit);
    if (res != NO_ERR) {
        return res;
    }

    res =
        agt_cb_register_callback(y_silsa_sethook_example_M_silsa_sethook_example,
        (const xmlChar *)"/sa-sethook-ex:interfaces/sa-sethook-ex:interface/sa-sethook-ex:hook-node",
                                 y_silsa_sethook_example_R_silsa_sethook_example,
                                 silsa_sethook_example_edit);
    if (res != NO_ERR) {
        return res;
    }

    res =
        agt_cb_register_callback(y_silsa_sethook_example_M_silsa_sethook_example,
        (const xmlChar *)"/sa-sethook-ex:interfaces/sa-sethook-ex:interface/sa-sethook-ex:state",
                                 y_silsa_sethook_example_R_silsa_sethook_example,
                                 silsa_sethook_example_edit);
    if (res != NO_ERR) {
        return res;
    }

    res =
        agt_cb_register_callback(y_silsa_sethook_example_M_silsa_sethook_example,
        (const xmlChar *)"/sa-sethook-ex:interfaces/sa-sethook-ex:interface/sa-sethook-ex:state/sa-sethook-ex:admin-state",
                                 y_silsa_sethook_example_R_silsa_sethook_example,
                                 silsa_sethook_example_edit);
    if (res != NO_ERR) {
        return res;
    }

    res =
        agt_cb_register_callback(y_silsa_sethook_example_M_silsa_sethook_example,
                (const xmlChar *)"/sa-sethook-ex:interfaces/sa-sethook-ex:status",
                                 y_silsa_sethook_example_R_silsa_sethook_example,
                                 silsa_sethook_example_edit);
    if (res != NO_ERR) {
        return res;
    }

    res =
        agt_cb_register_callback(y_silsa_sethook_example_M_silsa_sethook_example,
                                 (const xmlChar *)"/sa-sethook-ex:trigger",
                                 y_silsa_sethook_example_R_silsa_sethook_example,
                                 silsa_sethook_example_edit);
    if (res != NO_ERR) {
        return res;
    }

    /* put your module initialization code here */

    /* Set Hook callback registration */
    res = agt_cb_sa_hook_register((const xmlChar *)"/sa-sethook-ex:trigger",
                                  AGT_HOOKFMT_NODE,
                                  AGT_HOOK_TYPE_SETHOOK,
                                  sethook_callback);
    if (res != NO_ERR) {
        return res;
    }

    return res;

} /* y_silsa_sethook_example_init */

/********************************************************************
* FUNCTION y_silsa_sethook_example_init2
*
* SIL init phase 2: non-config data structures
* Called after running config is loaded
*
* RETURNS:
*     error status
********************************************************************/
status_t y_silsa_sethook_example_init2 (void)
{
    status_t res = NO_ERR;

    /* put your init2 code here */

    return res;

} /* y_silsa_sethook_example_init2 */

/********************************************************************
* FUNCTION y_silsa_sethook_example_cleanup
*    cleanup the server instrumentation library
*
********************************************************************/
void y_silsa_sethook_example_cleanup (void)
{

    agt_cb_unregister_callbacks(
        y_silsa_sethook_example_M_silsa_sethook_example,
        (const xmlChar *)"/sa-sethook-ex:interfaces");

    agt_cb_unregister_callbacks(
        y_silsa_sethook_example_M_silsa_sethook_example,
        (const xmlChar *)"/sa-sethook-ex:interfaces/sa-sethook-ex:interface");

    agt_cb_unregister_callbacks(
        y_silsa_sethook_example_M_silsa_sethook_example,
        (const xmlChar *)"/sa-sethook-ex:interfaces/sa-sethook-ex:interface/sa-sethook-ex:state");

    agt_cb_unregister_callbacks(
        y_silsa_sethook_example_M_silsa_sethook_example,
        (const xmlChar *)"/sa-sethook-ex:trigger");

    /* put your cleanup code here */


    /* Set Hook callback clean up */
    agt_cb_unregister_callbacks(y_silsa_sethook_example_M_silsa_sethook_example,
                                (const xmlChar *)"/sa-sethook-ex:trigger");


} /* y_silsa_sethook_example_cleanup */

/* END SIL-SA silsa_sethook_example.c */
